/*
 * Copyright (c) 2016 Intel Corporation
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

/**
 * @file main runtime loop for Gem.
 */

// MAIN LIBS
#include <zephyr.h>
#include <sys/printk.h>
#include <device.h>
#include <string.h>
#include <date_time.h>
#include <stdio.h>

// LOGGING
#include <logging/log.h>
#include <logging/log_ctrl.h>

// PERIPERHALS
#include <drivers/pwm.h>
#include <drivers/i2c.h>
#include <drivers/spi.h>
#include <drivers/gpio.h>
#include <drivers/led_strip.h>

// HAL Drivers
#include <pcf8575.h>
#include <veml7700.h>
#include <status_led.h>
#include <ws2812_driver.h>
#include <networking.h>
#include <nvs.h>
#include <update.h>
#include <sys/reboot.h>
#include <semver.h>

// DFU
#include <dfu/mcuboot.h>
#include <dfu/dfu_target_mcuboot.h>
#include <modem/nrf_modem_lib.h>
#include <net/fota_download.h>

// RUNTIME CONTROLLERS
#include <server_controller.h>
#include <ui_controller.h>

// version 1.0.0
#include "config.h"
#include <drivers/watchdog.h>
#include <task_wdt/task_wdt.h>

// Check every minute to see if the heartbeat has been flatline
#define HEARTBEAT_TIMEOUT_CHECK 1000*60
#define WDT_NODE DT_COMPAT_GET_ANY_STATUS_OKAY(nordic_nrf_watchdog)

LOG_MODULE_REGISTER(main);

void main(void)
{

        uint32_t Reset_reason = NRF_POWER->RESETREAS;
        LOG_INF("Previous Reset Reason: %d", Reset_reason);
        NRF_POWER->RESETREAS = 0; 

        LOG_INF("Booting FW Version: %s", log_strdup(FW_VERSION));

	struct device *gpio_dev;

	gpio_dev = device_get_binding("GPIO_0");

        // Turns on the 5V rail
        gpio_pin_configure(gpio_dev, 6, GPIO_OUTPUT); //p0.06
        gpio_pin_set(gpio_dev, 6, 0);

        const struct device *hw_wdt_dev = DEVICE_DT_GET(WDT_NODE);

        if (!device_is_ready(hw_wdt_dev)) {
	  printk("Hardware watchdog %s is not ready; ignoring it.\n",
		hw_wdt_dev->name);
	  hw_wdt_dev = NULL;
        }

        task_wdt_init(hw_wdt_dev);

        int task_wdt_id = task_wdt_add(2000, NULL, NULL);

        int attempts = 0; 

        // 3x attempts to connect to LTE network, then retry again every minute 
        do {
          LOG_INF("Connecting to server (attempt: %d)", attempts);
          connect_to_server();
          attempts++;
        } while ( (!get_server_status()) && (attempts < 3) );
        
        task_wdt_feed(task_wdt_id); 

	while (1) {      
          task_wdt_feed(task_wdt_id); 
          k_sleep(K_SECONDS(1U));
        }
}
